﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

// programmer: Bjørk Boye Busch, EAL

namespace SodukoSolverClassLibrary
{
    // Method are all made public, so they can be used for your own testing and solving as stand alone
    // Board is in this solution parameter for posibility to be used alone
    // board must be 9x9 elements
    public static class SodukoSolver
    {
        // row and col is position to be tested
        public static bool IsSmallBoxValid(int[,] board, int row, int col)
        {
            // validation of box 3x3
            int startBoxRow = (row / 3) * 3;
            int startBoxCol = (col / 3) * 3;
            bool[] usedNumber = { false, false, false, false, false, false, false, false, false, false };  // number 0-9 
            for (int r = startBoxRow; r < startBoxRow + 3; ++r)
            {
                for (int c = startBoxCol; c < startBoxCol + 3; ++c)
                {
                    int number = board[r, c];
                    if (number != 0)
                    {
                        if (usedNumber[number])
                            return false;
                        else
                            usedNumber[number] = true;
                    }
                }
            }
            return true;
        }

        // row and col is position to be tested
        public static bool IsRowValid(int[,] board, int row, int col)
        {
            // validation of row
            bool[] usedNumber = { false, false, false, false, false, false, false, false, false, false };  // number 0-9 
            for (int c = 0; c < 9; ++c)
            {
                int number = board[row, c];
                if (number != 0)
                {
                    if (usedNumber[number])
                        return false;
                    else
                        usedNumber[number] = true;
                }
            }
            return true;
        }
        // row and col is position to be tested
        public static bool IsColumnValid(int[,] board, int row, int col)
        {
            // validation of col
            bool[] usedNumber = { false, false, false, false, false, false, false, false, false, false };  // number 0-9 
            for (int r = 0; r < 9; ++r)
            {
                int number = board[r, col];
                if (number != 0)
                {
                    if (usedNumber[number])
                        return false;
                    else
                        usedNumber[number] = true;
                }
            }
            return true;
        }
        public static bool IsPositionValid(int[,] board, int row, int col)
        {
            return (IsSmallBoxValid(board, row, col) && IsRowValid(board, row, col) && IsColumnValid(board, row, col));
        }

        // check all by doing it diagonalt for few compare
        public static bool IsAllValid(int[,] board)
        {
            for (int n = 0; n < 9; ++n)
            {
                if (IsPositionValid(board, n, n))
                    return false;
            }
            return true;
        }

        public static bool SolvePart(int[,] board, int n)
        {
            if (n >= 81)    // no position left => no problem
                return true;
            int row = n / 9;
            int col = n % 9;
            if (board[row, col] == 0)
            {
                for (int number = 1; number < 10; ++number)
                {
                    board[row, col] = number;       // lets try one
                    if (IsPositionValid(board, row, col))     // if ok - try more
                    {
                        if (SolvePart(board, n + 1))    // work for rest ?
                            return true;
                    }
                }
                board[row, col] = 0;    // back to start
                return false;   // no solution from here
            }
            else
                return SolvePart(board, n + 1);     // skip and get solution from rest
        }

        public static bool Solve(int[,] board)
        {
            return SolvePart(board, 0);
        }
        public static string Board2String(int[,] board)
        {
            StringBuilder str = new StringBuilder(400);
            for (int r = 0; r < 9; ++r)
            {
                for (int c = 0; c < 9; ++c)
                {
                    int number = board[r, c];
                    if (number > 0)
                        str.Append(number + " ");
                    else
                        str.Append(". ");
                }
                str.Append(Environment.NewLine);
            }
            return str.ToString();
        }
    }
}